#!/bin/bash
#
# Helper scripts for playing with UPS-Coordinates
#

# Set the following paths appropriately
UPS_INDEL=/home/len/dev/ups-indel/ups_indel
RTG=/rtgshare/tools/rtg-tools/rtg
BEDTOOLS=/rtgshare/tools/bedtools/bin/bedtools
REF_FA=/rtgshare/data/human/ref/1000g_v37_phase2/hs37d5.fasta

# VCFs are assumed to correspond to the naming convention of the above REF_FA
# The script also wants awk/parallel/sort/cut/uniq etc.


THIS_SCRIPT=$0
THREADS=4

# Create a uvcf corresponding to a gzipped VCF, in the same directory
# Uses a single invocation of ups_indel. Due to a bug in the current
# version of ups_indel, this function doesn't work well with VCF files
# that containing calls on multiple chrs. See make_uvcf_chrs.
function make_uvcf_single() {
    local cvcf=$(basename "$1")
    local vcf="${cvcf/%.gz/}"
    if [[ "$vcf" == "$cvcf" ]]; then
        echo "Input is not .vcf.gz: $cvcf" >&2
        exit 1
    fi
    local dir=$(dirname "$1")
    local uvcf="${vcf/%.vcf/}"
    if [[ ! -f "$dir/${uvcf}.uvcf" ]]; then # Doesn't already exist, so we need to make it
        (
            # For some reason ups_indel doesn't like running on files that aren't in the current working directory
            # So jump through some hoops here
            cd "$dir"
            zcat -f "${cvcf}" >"${vcf}"
            $UPS_INDEL $REF_FA "${vcf}" "${uvcf}" || exit 1
            rm "${vcf}"
        )
    fi
    echo "$dir/${uvcf}.uvcf"
}

# Create a uvcf corresponding to a gzipped VCF, in the same directory
# This version handles VCF files containing calls on multiple chrs, by
# calling ups_indel separately on temporary per-chromosome VCFs.
function make_uvcf_chrs() {
    local cvcf=$(basename "$1")
    local vcf="${cvcf/%.gz/}"
    if [[ "$vcf" == "$cvcf" ]]; then
        echo "Input is not .vcf.gz: $cvcf" >&2
        exit 1
    fi
    local dir=$(dirname "$1")
    local prefix="${vcf/%.vcf/}"
    if [[ ! -f "$dir/${prefix}.uvcf" ]]; then # Doesn't already exist, so we need to make it
        echo "Getting used chromosome list from VCF" >&2
        local chrs=$($RTG extract "$1" | cut -f 1 | uniq)
        echo "Extracting VCFs for "$chrs >&2
        for chra in $chrs; do
            echo "$RTG vcffilter -i ${1} -o tmp_${prefix}_${chra}.vcf.gz --region $chra"
        done | parallel -j $THREADS 1>&2 || exit 1
        echo "Creating uVCFs for "$chrs >&2
        for chrb in $chrs; do
            echo "$THIS_SCRIPT make_uvcf_single tmp_${prefix}_${chrb}.vcf.gz"
        done | parallel -j $THREADS 1>&2 || exit 1
        echo "Combining uVCFs" >&2
        cat "tmp_${prefix}"_*.uvcf | grep -v "^#\|^CHROM" >"${prefix}.uvcf" || exit 1
        rm "tmp_${prefix}_${chr}"* || exit 1
    fi
    echo "$dir/${prefix}.uvcf"
}

# Delegate through to our implementation that splits on chromosomes
function make_uvcf() {
    make_uvcf_chrs "$1"
}

# Extract the UPS coordinates from a uvcf file as BED
function ups_coords() {
    local uvcf=$1
    grep -v "^#\|^CHROM" "$uvcf" | cut -f 1,8 | grep -v "N/A" | tr '][ ' '\t\t\t' | awk -F'\t' '{printf "%s\t%d\t%d\t%s\n",$1,$3-1,$5,$2}' | sort -k1,1 -k2,2n
}

# Find the UPS coordinates that are contained in both VCF files (i.e. indels in common)
function ups_indel_pair() {
    local vcf1=$1
    local vcf2=$2
    local uvcf1=$(make_uvcf "$vcf1")
    local uvcf2=$(make_uvcf "$vcf2")
    (ups_coords "$uvcf1"; ups_coords "$uvcf2") | sort | uniq -d | sort -k1,1 -k2,2n
}

# Find the UPS coordinates that are contained only in the first VCF file
function ups_indel_infirst() {
    local vcf1=$1
    local vcf2=$2
    local uvcf1=$(make_uvcf "$vcf1")
    local uvcf2=$(make_uvcf "$vcf2")
    (ups_coords "$uvcf1"; ups_coords "$uvcf2"; ups_coords "$uvcf2") | sort | uniq -u | sort -k1,1 -k2,2n
}

# Find the UPS coordinates that are contained only in the second VCF file
function ups_indel_insecond() {
    local vcf1=$1
    local vcf2=$2
    ups_indel_infirst "${vcf2}" "${vcf1}"
}

# Given a truth VCF and corresponding high-confidence regions, find
# cases where indels in the VCF cross high confidence region boundaries.
function gold_standard_unsafe() {
    local truthvcf=$1
    local truthbed=$2

    # Make UVCF from the truth
    local truthuvcf=$(make_uvcf "${truthvcf}")
    local basename=$(basename "$truthvcf")
    local vcfprefix="${basename/%.gz/}"
    vcfprefix="${vcfprefix/%.vcf/}"

    echo "Getting UPS-Coordinates spanning more than a base to BED"
    # Create in current directory
    [[ -f "${vcfprefix}.uvcf.bed.gz" ]] || ups_coords "${truthuvcf}" | awk -F'\t' '$3-$2>1{print}' | $RTG bgzip - >"${vcfprefix}.uvcf.bed.gz" && $RTG index -f bed "${vcfprefix}.uvcf.bed.gz" || exit 1

    echo "Making confident regions \"boundaries\" BED"
    # Create in current directory
    local bedprefix=$(basename "$truthbed")
    bedprefix="${bedprefix/%.gz/}"
    bedprefix="${bedprefix/%.bed/}"
    [[ -f "${bedprefix}.boundaries.bed.gz" ]] || zcat -f "${truthbed}" | awk -F'\t' '{printf "%s\t%d\t%d\n%s\t%d\t%d\n",$1,$2,$2+1,$1,$3-1,$3}' | $RTG bgzip - >"${bedprefix}.boundaries.bed.gz" && $RTG index -f bed "${bedprefix}.boundaries.bed.gz" || exit 1

    echo "Make intersection BED giving locations where UPS-Coordinates span confident region boundaries"
    [[ -f "${bedprefix}.boundaries.unsafe.bed.gz" ]] || $BEDTOOLS intersect -a "${vcfprefix}.uvcf.bed.gz" -b "${bedprefix}.boundaries.bed.gz" | sort -k1,1 -k2,2n | $RTG bgzip - >"${bedprefix}.boundaries.unsafe.bed.gz" && $RTG index -f bed "${bedprefix}.boundaries.unsafe.bed.gz" || exit 1
}



function _main() {
    if [[ ! "$1" ]] || [[ "$1" == "help" ]]; then
        cat<<EOF
USAGE: Give name of function to run, e.g.:

EOF
        sed -n "s/^function \([^_][^()]\+\)(.*/\t\1/p" <$0
        echo
        exit 1

    else
        "$@"
        exit
    fi
}
_main "$@"
